@@ -1,10 +1,13 @@
-
 import os
 
 import pytest
 
 from boltons.dictutils import OMD
 from boltons.iterutils import (first,
+                               pairwise,
+                               pairwise_iter,
+                               windowed,
+                               windowed_iter,
                                remap,
                                research,
                                default_enter,
@@ -21,7 +24,7 @@ even = lambda x: isint(x) and x % 2 == 0
 is_meaning_of_life = lambda x: x == 42
 
 
-class TestFirst(object):
+class TestFirst:
     def test_empty_iterables(self):
         """
         Empty iterables return None.
@@ -57,7 +60,7 @@ class TestFirst(object):
         assert first(l, key=is_meaning_of_life) is None
 
 
-class TestRemap(object):
+class TestRemap:
     # TODO: test namedtuples and other immutable containers
 
     def test_basic_clone(self):
@@ -279,8 +282,8 @@ class TestRemap(object):
                  'dads': [{'name': 'Kurt',
                            'interests': ['python', 'recursion']}]}]
 
-        ref = set(['python', 'recursion', 'biking', 'museums',
-                   'pears', 'theater', 'manga'])
+        ref = {'python', 'recursion', 'biking', 'museums',
+                   'pears', 'theater', 'manga'}
 
         remap(orig, enter=enter)
         assert all_interests == ref
@@ -337,7 +340,7 @@ class TestRemap(object):
 
     def test_remap_set(self):
         # explicit test for sets to make sure #84 is covered
-        s = set([1, 2, 3])
+        s = {1, 2, 3}
         assert remap(s) == s
 
         fs = frozenset([1, 2, 3])
@@ -356,7 +359,7 @@ class TestRemap(object):
         return
 
 
-class TestGetPath(object):
+class TestGetPath:
     def test_depth_one(self):
         root = ['test']
         assert get_path(root, (0,)) == 'test'
@@ -551,3 +554,25 @@ def test_strip():
     assert strip([0,0,0,1,0,2,0,3,0,0,0],0) == [1,0,2,0,3]
     assert strip([]) == []
 
+
+def test_pairwise_filled():
+    assert pairwise(range(4)) == [(0, 1), (1, 2), (2, 3)]
+    assert pairwise(range(4), end=None) == [(0, 1), (1, 2), (2, 3), (3, None)]
+
+    assert pairwise([]) == []
+    assert pairwise([1], end=None) == [(1, None)]
+
+    assert list(pairwise_iter(range(4))) == [(0, 1), (1, 2), (2, 3)]
+    assert list(pairwise_iter(range(4), end=None)) == [(0, 1), (1, 2), (2, 3), (3, None)]
+
+
+def test_windowed_filled():
+    assert windowed(range(4), 3) == [(0, 1, 2), (1, 2, 3)]
+    assert windowed(range(4), 3, fill=None) == [(0, 1, 2), (1, 2, 3), (2, 3, None), (3, None, None)]
+
+    assert windowed([], 3) == []
+    assert windowed([], 3, fill=None) == []
+    assert windowed([1, 2], 3, fill=None) == [(1, 2, None), (2, None, None)]
+
+    assert list(windowed_iter(range(4), 3)) == [(0, 1, 2), (1, 2, 3)]
+    assert list(windowed_iter(range(4), 3, fill=None)) == [(0, 1, 2), (1, 2, 3), (2, 3, None), (3, None, None)]
