@@ -1,11 +1,6 @@
-# -*- coding: utf-8 -*-
 import json
 import sys
-
-try:
-    from cStringIO import StringIO
-except:
-    from io import StringIO
+from io import StringIO
 
 from boltons.tbutils import (TracebackInfo,
                              ExceptionInfo,
